from __future__ import annotations
import json
import datetime as dt
import inspect
from typing import overload, cast


class KeeWeeDB(dict):
    def __missing__(self, kee):
        wee = self[kee] = type(self)()
        return wee


class KeeWee:
    _store = {}

    def __init__(self, blame: bool = False):
        self.blame = blame

    def __set_name__(self, owner: type[object], name: str) -> None:
        self.public_name = name
        self.private_name = '_' + name
        self._store[owner.__name__] = {}
        self._store[owner.__name__][name] = {}

    def __set__(self, obj: object, value: int) -> None:
        obj.__dict__[self.private_name] = value

        owner = obj.__class__.__name__
        instance_name = str(obj)
        if not self._store[owner][self.public_name].get(instance_name):
            self._store[owner][self.public_name][instance_name] = {}
        if self.blame:
            mutator = inspect.getouterframes(inspect.currentframe(), 2)[1][3]
            self._store[owner][self.public_name][instance_name][dt.datetime.now().time().isoformat()] = (value, mutator)
        else:
            self._store[owner][self.public_name][instance_name][dt.datetime.now().time().isoformat()] = value

    @overload
    def __get__(self, obj: None, obj_type: None) -> KeeWee:
        ...

    @overload
    def __get__(self, obj: object, obj_type: type[object]) -> int:
        ...

    def __get__(self, obj: object | None, obj_type: type[object] | None = None) -> KeeWee | int:
        if obj is None:
            return self
        return cast(int, obj.__dict__.get(self.private_name))

    @classmethod
    def dump(cls, file_name: str):
        with open(file_name, "w") as f:
            json.dump(cls._store, f)

    @classmethod
    def dumps(cls):
        return json.dumps(cls._store)

    @classmethod
    def dumpd(cls):
        return cls._store



import unittest
from dataclasses import dataclass, field

from keewee import KeeWee


@dataclass
class PokemonTrainer:
    name: str
    skill_level: int | KeeWee = field(default=KeeWee(), repr=False)


class MyTestCase(unittest.TestCase):

    def test_something(self):
        def add_1(pokemon_trainer: PokemonTrainer) -> None:
            pokemon_trainer.skill_level += 1

        andrew = PokemonTrainer(name="Andrea Taylor", skill_level=0)
        self.assertEqual(andrew.skill_level, 0)
        andrew.skill_level = 1
        self.assertEqual(andrew.skill_level, 1)

        ash = PokemonTrainer(name="Ash Ketchum", skill_level=0)

        import random
        for _ in range(4):
            ash.skill_level = random.randint(1, 10)

        add_1(andrew)
        self.assertEqual(andrew.skill_level, 2)
        print(KeeWee.dump("tests.json"))

    def test_for_readme(self):
        import random
        from pprint import pprint

        ash = PokemonTrainer(name="Ash Ketchum", skill_level=0)

        for _ in range(10):
            ash.skill_level = random.randint(1, 10)

        pprint(
            KeeWee.dumpd()
        )


if __name__ == '__main__':
    unittest.main()



import unittest

from keewee import KeeWee


class Employee:
    skill_level = KeeWee()

    def __init__(self, name: str, skill_level: int):
        self.name = name
        self.skill_level = skill_level

    def __str__(self):
        return f"Employee( name: {self.name})"


class MyTestCase(unittest.TestCase):

    def test_something(self):
        andrew = Employee(name="Andrea Taylor", skill_level=0)
        self.assertEqual(andrew.skill_level, 0)
        andrew.skill_level = 1
        self.assertEqual(andrew.skill_level, 1)

        KeeWee.dump(file_name="andrea.json")


if __name__ == '__main__':
    unittest.main()
